@@ -4,16 +4,16 @@
  * Procedures for traversing the hierarchical representation
  * of a circuit.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -32,7 +32,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/utils.h"
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -77,9 +77,13 @@ efHierSrUses(hc, func, cdata)
     HierContext newhc;
     Transform t;
     Use *u;
+    HashSearch hs;
+    HashEntry *he;
 
-    for (u = hc->hc_use->use_def->def_uses; u; u = u->use_next)
+    HashStartSearch(&hs);
+    while (he = HashNext(&hc->hc_use->use_def->def_uses, &hs))
     {
+	u = (Use *)HashGetValue(he);
 	newhc.hc_use = u;
 	if (!IsArray(u))
 	{
@@ -115,7 +119,7 @@ efHierSrUses(hc, func, cdata)
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -236,6 +240,9 @@ EFHierSrDefs(hc, func, cdata)
 {
     HierContext newhc;
     Use *u;
+    int retval;
+    HashSearch hs;
+    HashEntry *he;
 
     if (func == NULL)
     {
@@ -250,8 +257,10 @@ EFHierSrDefs(hc, func, cdata)
 	hc->hc_use->use_def->def_flags |= DEF_PROCESSED;
     }
 
-    for (u = hc->hc_use->use_def->def_uses; u; u = u->use_next)
+    HashStartSearch(&hs);
+    while (he = HashNext(&hc->hc_use->use_def->def_uses, &hs))
     {
+	u = (Use *)HashGetValue(he);
 	newhc.hc_use = u;
 	newhc.hc_hierName = NULL;
 	GeoTransTrans(&u->use_trans, &hc->hc_trans, &newhc.hc_trans);
@@ -261,7 +270,14 @@ EFHierSrDefs(hc, func, cdata)
     if (func == NULL)
 	return 0;
     else
-	return ((*func)(hc, cdata));
+    {
+	/* Clear DEF_PROCESSED for the duration of running the function */
+
+	hc->hc_use->use_def->def_flags &= ~DEF_PROCESSED;
+	retval = (*func)(hc, cdata);
+	hc->hc_use->use_def->def_flags |= DEF_PROCESSED;
+	return retval;
+    }
 }
 
 /*----------------------------------------------------------------------*/
@@ -381,6 +397,7 @@ efHierDevKilled(hc, dev, prefix)
 
     for (n = 0; n < dev->dev_nterm; n++)
     {
+	if (dev->dev_terms[n].dterm_node == NULL) continue;
 	suffix = dev->dev_terms[n].dterm_node->efnode_name->efnn_hier;
 	he = HashLookOnly(&efNodeHashTable, (char *)suffix);
 	if (he  && (nn = (EFNodeName *) HashGetValue(he))
@@ -455,6 +472,8 @@ efHierVisitDevs(hc, ca)
 {
     Def *def = hc->hc_use->use_def;
     Dev *dev;
+    HashSearch hs;
+    HashEntry *he;
     float scale;
 
     /*
@@ -466,8 +485,10 @@ efHierVisitDevs(hc, ca)
     scale = (efScaleChanged && def->def_scale != 1.0) ? def->def_scale : 1.0;
 
     /* Visit all devices */
-    for (dev = def->def_devs; dev; dev = dev->dev_next)
+    HashStartSearch(&hs);
+    while (he = HashNext(&def->def_devs, &hs))
     {
+	dev = (Dev *)HashGetValue(he);
 	if (efHierDevKilled(hc, dev, hc->hc_hierName))
 	    continue;
 
