@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMakeRes.c,v 1.2 2009/01/30 03:11:15 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMakeRes.c,v 1.3 2010/06/24 12:37:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -24,18 +24,19 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/tech.h"
 #include "textio/txcommands.h"
 #include "resis/resis.h"
+#include "cif/CIFint.h"
 
 /* Forward declarations */
-bool ResCalcNearTransistor();
+bool ResCalcNearDevice();
 bool ResCalcNorthSouth();
 bool ResCalcEastWest();
 
-
+
 /*
  *--------------------------------------------------------------------------
  *
  * ResCalcTileResistance-- Given a set of partitions for a tile, the tile can
- *	be converted into resistors. To do this, nodes are sorted in the 
+ *	be converted into resistors. To do this, nodes are sorted in the
  *	direction of current flow. Resistors are created by counting squares
  *	between successive breakpoints. Breakpoints with the same coordinate
  *	are combined.
@@ -48,7 +49,7 @@ bool ResCalcEastWest();
  *
  */
 
-bool 
+bool
 ResCalcTileResistance(tile, junk, pendingList, doneList)
     Tile 	*tile;
     tileJunk 	*junk;
@@ -57,13 +58,13 @@ ResCalcTileResistance(tile, junk, pendingList, doneList)
 {
     int 	MaxX = MINFINITY, MinX = INFINITY;
     int		MaxY = MINFINITY, MinY = INFINITY;
-    int		transistor;
+    int		device;
     bool	merged;
     Breakpoint 	*p1;
-     
+
     merged = FALSE;
-    transistor = FALSE;
-     
+    device = FALSE;
+
     if ((p1 = junk->breakList) == NULL) return FALSE;
     for (; p1; p1 = p1->br_next)
     {
@@ -73,18 +74,18 @@ ResCalcTileResistance(tile, junk, pendingList, doneList)
 	if (x < MinX) MinX = x;
 	if (y > MaxY) MaxY = y;
 	if (y < MinY) MinY = y;
-	if (p1->br_this->rn_why == RES_NODE_TRANSISTOR)
+	if (p1->br_this->rn_why == RES_NODE_DEVICE)
 	{
-	    transistor = TRUE;
+	    device = TRUE;
 	}
     }
-     
+
     /* Finally, produce resistors for partition. Keep track of	*/
     /* whether or not the node was involved in a merge.		*/
-     
-    if (transistor)
+
+    if (device)
     {
-	merged |= ResCalcNearTransistor(tile, pendingList, doneList, &ResResList);
+	merged |= ResCalcNearDevice(tile, pendingList, doneList, &ResResList);
     }
     else if (MaxY-MinY > MaxX-MinX)
     {
@@ -95,7 +96,7 @@ ResCalcTileResistance(tile, junk, pendingList, doneList)
 	merged |= ResCalcEastWest(tile, pendingList, doneList, &ResResList);
     }
 
-    /* 
+    /*
      * For all the new resistors, propagate the resistance from the origin
      * to the new nodes.
      */
@@ -103,7 +104,7 @@ ResCalcTileResistance(tile, junk, pendingList, doneList)
     return(merged);
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -130,17 +131,17 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
     resNode	*currNode;
     float	rArea;
     tileJunk	*junk = (tileJunk *)tile->ti_client;
-     
+
     merged = FALSE;
     height = TOP(tile) - BOTTOM(tile);
 
-    /* 
+    /*
      * One Breakpoint? No resistors need to be made. Free up the first
      * breakpoint, then return.
      */
 
     p1 = junk->breakList;
-    if   (p1->br_next == NULL) 
+    if   (p1->br_next == NULL)
     {
 	p1->br_this->rn_float.rn_area += height * (LEFT(tile) - RIGHT(tile));
 	freeMagic((char *)p1);
@@ -151,9 +152,9 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
     /* re-sort nodes left to right. */
 
     ResSortBreaks(&junk->breakList, TRUE);
-     
-    /* 
-     * Eliminate breakpoints with the same X coordinate and merge 
+
+    /*
+     * Eliminate breakpoints with the same X coordinate and merge
      * their nodes.
      */
 
@@ -198,7 +199,7 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 		 freeMagic((char *)p1);
 	    }
 
-	    /* 
+	    /*
 	     * Was the node used in another junk or breakpoint?
 	     * If so, replace the old node with the new one.
 	     */
@@ -213,7 +214,7 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 	    }
        }
 
-       /* 
+       /*
         * If the X coordinates don't match, make a resistor between
         * the breakpoints.
         */
@@ -238,7 +239,6 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 	    resistor->rr_cl = (TOP(tile) + BOTTOM(tile)) >> 1;
 	    resistor->rr_width = height;
 
-#ifdef NONMANHATTAN
 	    if (IsSplit(tile))
 	    {
 		resistor->rr_tt = (SplitSide(tile)) ? SplitRightType(tile)
@@ -246,17 +246,13 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 		resistor->rr_status = RES_DIAGONAL;
 		resistor->rr_status |= (SplitDirection(tile)) ? RES_NS
 			: RES_EW;
-			
+
 	    }
 	    else
 	    {
 		resistor->rr_status = RES_EW;
 		resistor->rr_tt = TiGetTypeExact(tile);
 	    }
-#else
-	    resistor->rr_status = RES_EW;
-	    resistor->rr_tt = TiGetType(tile);
-#endif
 #ifdef ARIEL
 	    resistor->rr_csArea = height *
 				ExtCurStyle->exts_thick[resistor->rr_tt];
@@ -268,7 +264,7 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 	    resistor->rr_connection1->rn_float.rn_area += rArea;
 	    resistor->rr_connection2->rn_float.rn_area += rArea;
 	    resistor->rr_float.rr_area = 0;
-		    
+
 	    freeMagic((char *)p1);
 	}
     }
@@ -278,7 +274,7 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
     return(merged);
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -305,17 +301,17 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
     resNode	*currNode;
     float	rArea;
     tileJunk	*junk = (tileJunk *)tile->ti_client;
-     
+
     merged = FALSE;
     width = RIGHT(tile)-LEFT(tile);
 
-    /* 
+    /*
      * One Breakpoint? No resistors need to be made. Free up the first
      * breakpoint, then return.
      */
 
     p1 = junk->breakList;
-    if   (p1->br_next == NULL) 
+    if (p1->br_next == NULL)
     {
 	p1->br_this->rn_float.rn_area += width * (TOP(tile) - BOTTOM(tile));
      	freeMagic((char *)p1);
@@ -326,8 +322,8 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
     /* re-sort nodes south to north. */
     ResSortBreaks(&junk->breakList, FALSE);
 
-    /* 
-     * Eliminate breakpoints with the same Y coordinate and merge 
+    /*
+     * Eliminate breakpoints with the same Y coordinate and merge
      * their nodes.
      */
 
@@ -371,8 +367,8 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 	    	 ResMergeNodes(p2->br_this,p1->br_this,pendingList,doneList);
 		 freeMagic((char *)p1);
 	    }
-		    
-	    /* 
+
+	    /*
 	     * Was the node used in another junk or breakpoint?
 	     * If so, replace the old node with the new one.
 	     */
@@ -387,7 +383,7 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 	    }
 	}
 
-	/* 
+	/*
 	 * If the Y coordinates don't match, make a resistor between
 	 * the breakpoints.
 	 */
@@ -411,7 +407,6 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 	    p2->br_this->rn_re = element;
 	    resistor->rr_cl = (LEFT(tile) + RIGHT(tile)) >> 1;
 	    resistor->rr_width = width;
-#ifdef NONMANHATTAN
 	    if (IsSplit(tile))
 	    {
 		resistor->rr_tt = (SplitSide(tile)) ? SplitRightType(tile)
@@ -425,10 +420,6 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 		resistor->rr_status = RES_NS;
 		resistor->rr_tt = TiGetTypeExact(tile);
 	    }
-#else
-	    resistor->rr_status = RES_NS;
-	    resistor->rr_tt = TiGetType(tile);
-#endif
 #ifdef ARIEL
 	    resistor->rr_csArea = width
 			* ExtCurStyle->exts_thick[resistor->rr_tt];
@@ -449,15 +440,15 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
     return(merged);
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
- * ResCalcNearTransistor-- Calculating the direction of current flow near
- *	transistors is tricky because there are two adjoining regions with
- *	vastly different sheet resistances.  ResCalcNearTransistor is called
+ * ResCalcNearDevice-- Calculating the direction of current flow near
+ *	devices is tricky because there are two adjoining regions with
+ *	vastly different sheet resistances.  ResCalcNearDevice is called
  *	whenever a diffusion tile adjoining a real tile is found.  It makes
- *	a guess at the correct direction of current flow, removes extra 
+ *	a guess at the correct direction of current flow, removes extra
  *	breakpoints, and call either ResCalcEastWest or ResCalcNorthSouth
  *
  * Results:
@@ -469,75 +460,75 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
  */
 
 bool
-ResCalcNearTransistor(tile, pendingList, doneList, resList)
+ResCalcNearDevice(tile, pendingList, doneList, resList)
     Tile	*tile;
     resNode	**pendingList, **doneList;
     resResistor	**resList;
 
 {
      bool 		merged;
-     int		trancount,tranedge,deltax,deltay;
+     int		devcount,devedge,deltax,deltay;
      Breakpoint		*p1,*p2,*p3;
      tileJunk		*junk = (tileJunk *)tile->ti_client;
-     
-     
+
+
      merged = FALSE;
 
-     /* 
+     /*
         One Breakpoint? No resistors need to be made. Free up the first
      	breakpoint, then return.
      */
-     
-     if   (junk->breakList->br_next == NULL) 
+
+     if   (junk->breakList->br_next == NULL)
      {
      	  freeMagic((char *)junk->breakList);
 	  junk->breakList = NULL;
 	  return(merged);
      }
-     /* count the number of transistor breakpoints  */
+     /* count the number of device breakpoints  */
      /* mark which edge they connect to		    */
-     trancount = 0;
-     tranedge = 0;
+     devcount = 0;
+     devedge = 0;
      for (p1=junk->breakList; p1 != NULL;p1 = p1->br_next)
      {
-	  if (p1->br_this->rn_why == RES_NODE_TRANSISTOR)
+	  if (p1->br_this->rn_why == RES_NODE_DEVICE)
 	  {
-	       trancount++;
-	       if (p1->br_loc.p_x == LEFT(tile)) tranedge |= LEFTEDGE;
-	       else if (p1->br_loc.p_x == RIGHT(tile)) tranedge |= RIGHTEDGE;
-	       else if (p1->br_loc.p_y == TOP(tile)) tranedge |= TOPEDGE;
-	       else if (p1->br_loc.p_y == BOTTOM(tile)) tranedge |= BOTTOMEDGE;
+	       devcount++;
+	       if (p1->br_loc.p_x == LEFT(tile)) devedge |= LEFTEDGE;
+	       else if (p1->br_loc.p_x == RIGHT(tile)) devedge |= RIGHTEDGE;
+	       else if (p1->br_loc.p_y == TOP(tile)) devedge |= TOPEDGE;
+	       else if (p1->br_loc.p_y == BOTTOM(tile)) devedge |= BOTTOMEDGE;
 	  }
      }
-     /* use distance from transistor to next breakpoint as determinant     */
-     /* if there is only one transistor or if all the transitors are along */
+     /* use distance from device to next breakpoint as determinant     */
+     /* if there is only one device or if all the devices are along */
      /* the same edge.							   */
-     if (trancount == 1 		|| 
-        (tranedge & LEFTEDGE) == tranedge	||
-        (tranedge & RIGHTEDGE) == tranedge 	||
-        (tranedge & TOPEDGE) == tranedge 	||
-        (tranedge & BOTTOMEDGE) == tranedge)
+     if (devcount == 1 		||
+        (devedge & LEFTEDGE) == devedge	||
+        (devedge & RIGHTEDGE) == devedge 	||
+        (devedge & TOPEDGE) == devedge 	||
+        (devedge & BOTTOMEDGE) == devedge)
      {
 	  ResSortBreaks(&junk->breakList,TRUE);
           p2 = NULL;
           for (p1=junk->breakList; p1 != NULL;p1 = p1->br_next)
           {
-	       if (p1->br_this->rn_why == RES_NODE_TRANSISTOR)
+	       if (p1->br_this->rn_why == RES_NODE_DEVICE)
 	       {
 	            break;
 	       }
-	       if (p1->br_next != NULL && 
+	       if (p1->br_next != NULL &&
 	       	     (p1->br_loc.p_x != p1->br_next->br_loc.p_x ||
 	       	      p1->br_loc.p_y != p1->br_next->br_loc.p_y))
-	       
+
 	       {
 	       	    p2 = p1;
 	       }
           }
 	  deltax=INFINITY;
-	  for (p3 = p1->br_next; 
-	       p3 != NULL && 
-	       p3->br_loc.p_x == p1->br_loc.p_x && 
+	  for (p3 = p1->br_next;
+	       p3 != NULL &&
+	       p3->br_loc.p_x == p1->br_loc.p_x &&
 	       p3->br_loc.p_y == p1->br_loc.p_y; p3 = p3->br_next);
 	  if (p3 != NULL)
 	  {
@@ -589,22 +580,22 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
           p2 = NULL;
           for (p1=junk->breakList; p1 != NULL;p1 = p1->br_next)
           {
-	       if (p1->br_this->rn_why == RES_NODE_TRANSISTOR)
+	       if (p1->br_this->rn_why == RES_NODE_DEVICE)
 	       {
 	            break;
 	       }
-	       if (p1->br_next != NULL && 
+	       if (p1->br_next != NULL &&
 	       	     (p1->br_loc.p_x != p1->br_next->br_loc.p_x ||
 	       	      p1->br_loc.p_y != p1->br_next->br_loc.p_y))
-	       
+
 	       {
 	       	    p2 = p1;
 	       }
           }
 	  deltay=INFINITY;
-	  for (p3 = p1->br_next; 
-	       p3 != NULL && 
-	       p3->br_loc.p_x == p1->br_loc.p_x && 
+	  for (p3 = p1->br_next;
+	       p3 != NULL &&
+	       p3->br_loc.p_x == p1->br_loc.p_x &&
 	       p3->br_loc.p_y == p1->br_loc.p_y; p3 = p3->br_next);
 	  if (p3 != NULL)
 	  {
@@ -660,27 +651,27 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
 	  }
 
      }
-     /* multiple transistors connected to the partition */
-     else  
+     /* multiple devices connected to the partition */
+     else
      {
-     	  if (tranedge == 0)
+     	  if (devedge == 0)
 	  {
-	       TxError("Error in transistor current direction routine\n");
+	       TxError("Error in device current direction routine\n");
 	       return(merged);
 	  }
 	  /* check to see if the current flow is north-south		*/
 	  /* possible north-south conditions:				*/
-	  /* 1. there are transistors along the top and bottom edges    */
+	  /* 1. there are devices along the top and bottom edges    */
 	  /*    but not along the left or right			        */
-	  /* 2. there are transistors along two sides at right angles,  */
+	  /* 2. there are devices along two sides at right angles,  */
 	  /*    and the tile is wider than it is tall.			*/
-	  
-	  if ((tranedge & TOPEDGE)     && 
-	      (tranedge & BOTTOMEDGE)  &&
-	      !(tranedge & LEFTEDGE)   &&
-	      !(tranedge & RIGHTEDGE)  			||
-	      (tranedge & TOPEDGE || tranedge & BOTTOMEDGE) &&
-	      (tranedge & LEFTEDGE || tranedge & RIGHTEDGE) &&
+
+	  if ((devedge & TOPEDGE)     &&
+	      (devedge & BOTTOMEDGE)  &&
+	      !(devedge & LEFTEDGE)   &&
+	      !(devedge & RIGHTEDGE)  			||
+	      (devedge & TOPEDGE || devedge & BOTTOMEDGE) &&
+	      (devedge & LEFTEDGE || devedge & RIGHTEDGE) &&
 	      RIGHT(tile)-LEFT(tile) > TOP(tile)-BOTTOM(tile))
 	 {
                /* re-sort nodes south to north. */
@@ -689,7 +680,7 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
 	      /* eliminate duplicate S/D pointers */
 	      for (p1 = junk->breakList; p1 != NULL; p1 = p1->br_next)
 	      {
-	      	   if  (p1->br_this->rn_why == RES_NODE_TRANSISTOR &&
+	      	   if  (p1->br_this->rn_why == RES_NODE_DEVICE &&
 		       (p1->br_loc.p_y == BOTTOM(tile) ||
 		        p1->br_loc.p_y == TOP(tile)))
 		   {
@@ -697,7 +688,7 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
 			p2 = junk->breakList;
 			while ( p2 != NULL)
 			{
-			     if (p2->br_this == p1->br_this	&& 
+			     if (p2->br_this == p1->br_this	&&
 			         p2 != p1			&&
 				 p2->br_loc.p_y != BOTTOM(tile) &&
 				 p2->br_loc.p_y != TOP(tile))
@@ -730,7 +721,7 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
 	      /* eliminate duplicate S/D pointers */
 	      for (p1 = junk->breakList; p1 != NULL; p1 = p1->br_next)
 	      {
-	      	   if (p1->br_this->rn_why == RES_NODE_TRANSISTOR &&
+	      	   if (p1->br_this->rn_why == RES_NODE_DEVICE &&
 		       (p1->br_loc.p_x == LEFT(tile) ||
 		        p1->br_loc.p_x == RIGHT(tile)))
 		   {
@@ -738,7 +729,7 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
 			p2 = junk->breakList;
 			while ( p2 != NULL)
 			{
-			     if (p2->br_this == p1->br_this	&& 
+			     if (p2->br_this == p1->br_this	&&
 			         p2 != p1			&&
 				 p2->br_loc.p_x != LEFT(tile) &&
 				 p2->br_loc.p_x != RIGHT(tile))
@@ -769,14 +760,14 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
      }
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
  * ResDoContacts-- Add node (or nodes) for a contact.  If there are contact
  *	resistances, also add a resistor.
  *
- * Results: 
+ * Results:
  *	None.
  *
  * Side Effects: Creates nodes and resistors
@@ -793,13 +784,14 @@ ResDoContacts(contact, nodes, resList)
     resNode	 *resptr;
     cElement	 *ccell;
     int		 tilenum, squaresx, squaresy, viawidth;
-    int 	 minside, spacing, border;
+    int 	 minside, spacing, border, cscale;
     float	 squaresf;
     resResistor	 *resistor;
     resElement	 *element;
     static int	 too_small = 1;
-     
+
     minside = CIFGetContactSize(contact->cp_type, &viawidth, &spacing, &border);
+    cscale = CIFCurStyle->cs_scaleFactor;
 
     if ((ExtCurStyle->exts_viaResist[contact->cp_type] == 0) || (viawidth == 0))
     {
@@ -828,7 +820,8 @@ ResDoContacts(contact, nodes, resList)
     }
     else
     {
-	if ((contact->cp_width < minside) || (contact->cp_height < minside))
+	if (((contact->cp_width * cscale) < minside) ||
+		    ((contact->cp_height * cscale) < minside))
 	{
 	    if (too_small)
 	    {
@@ -842,13 +835,13 @@ ResDoContacts(contact, nodes, resList)
 	else
 	{
 	    viawidth += spacing;
-	    squaresf = (float)(contact->cp_width - minside) / (float)viawidth;
+	    squaresf = (float)((contact->cp_width * cscale) - minside) / (float)viawidth;
 	    squaresf *= ExtCurStyle->exts_unitsPerLambda;
 	    squaresf /= (float)viawidth;
 	    squaresx = (int)squaresf;
 	    squaresx++;
 
-	    squaresf = (float)(contact->cp_height - minside) / (float)viawidth;
+	    squaresf = (float)((contact->cp_height * cscale) - minside) / (float)viawidth;
 	    squaresf *= ExtCurStyle->exts_unitsPerLambda;
 	    squaresf /= (float)viawidth;
 	    squaresy = (int)squaresf;
@@ -863,14 +856,14 @@ ResDoContacts(contact, nodes, resList)
 	    resptr = (resNode *) mallocMagic((unsigned) (sizeof(resNode)));
 	    InitializeNode(resptr,x,y,RES_NODE_CONTACT);
 	    ResAddToQueue(resptr,nodes);
-	  
+
  	    /* add contact pointer to node  */
 
 	    ccell = (cElement *) mallocMagic((unsigned) (sizeof(cElement)));
 	    ccell->ce_nextc = resptr->rn_ce;
 	    resptr->rn_ce = ccell;
 	    ccell->ce_thisc = contact;
-	       
+
 	    contact->cp_cnode[tilenum] = resptr;
 	    NEWBREAK(resptr, tile, contact->cp_center.p_x,
 			contact->cp_center.p_y, &contact->cp_rect);
@@ -886,7 +879,7 @@ ResDoContacts(contact, nodes, resList)
 	        (*resList) = resistor;
 	        resistor->rr_connection1 = contact->cp_cnode[tilenum-1];
 	        resistor->rr_connection2 = contact->cp_cnode[tilenum];
-		    
+
 	        element = (resElement *) mallocMagic((unsigned) (sizeof(resElement)));
 	        element->re_nextEl = contact->cp_cnode[tilenum-1]->rn_re;
 	        element->re_thisEl = resistor;
@@ -896,11 +889,11 @@ ResDoContacts(contact, nodes, resList)
 	        element->re_thisEl = resistor;
 	        contact->cp_cnode[tilenum]->rn_re = element;
 
-		/* Need to figure out how to handle the multiple nodes	*/	
+		/* Need to figure out how to handle the multiple nodes	*/
 		/* and multiple resistors necessary to determine the 	*/
 		/* correct geometry for the geometry extractor.  For	*/
 		/* now, extract as one big glob.			*/
-	
+
 		/* rr_cl doesn't need to represent centerline; use for	*/
 		/* # squares in y direction instead; use rr_width for	*/
 		/* # squares in x direction.				*/
@@ -973,4 +966,4 @@ ResSortBreaks(masterlist, xsort)
 	}
     }
 }
-	
+
